/*
 This file is part of GNU Taler
 (C) 2022 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 *
 * @author Sebastian Javier Marchano (sebasjm)
 */
import { strings } from "./i18n/strings.js";

import * as pages from "./pages/index.stories.js";

import { renderStories } from "@gnu-taler/web-util/browser";

import "./scss/main.css";
import { h, ComponentChildren, FunctionComponent, VNode } from "preact";
import { ExchangeApiContextTesting } from "./context/config.js";

function main(): void {
  renderStories(
    { pages },
    {
      strings,
      getWrapperForGroup
    },
  );
}

function getWrapperForGroup(): FunctionComponent {
  return function All({ children }: { children?: ComponentChildren }): VNode {
    return <ExchangeApiContextTesting
      config={{
        currency: "ARS",
        currency_specification: {
          alt_unit_names: {},
          name: "ARS",
          num_fractional_input_digits: 2,
          num_fractional_normal_digits: 2,
          num_fractional_trailing_zero_digits: 2
        },
        name: "taler-exchange",
        supported_kyc_requirements: [],
        version: "asd",
      }}>
      {children}
    </ExchangeApiContextTesting>
  }
}

if (document.readyState === "loading") {
  document.addEventListener("DOMContentLoaded", main);
} else {
  main();
}
