/*
 This file is part of GNU Taler
 (C) 2022-2024 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

import { ComponentChildren, createContext, h, VNode } from "preact";
import { useContext } from "preact/hooks";
import {
  BackendStateHandler,
  defaultState,
  useBackendState,
} from "../hooks/backend.js";

/**
 *
 * @author Sebastian Javier Marchano (sebasjm)
 */

export type Type = BackendStateHandler;

const initial: Type = {
  state: defaultState,
  logOut() {
    null;
  },
  expired() {
    null;
  },
  logIn(_info) {
    null;
  },
};
const Context = createContext<Type>(initial);

export const useBackendContext = (): Type => useContext(Context);

export const BackendStateProvider = ({
  children,
}: {
  children: ComponentChildren;
}): VNode => {
  const value = useBackendState();

  return h(Context.Provider, {
    value,
    children,
  });
};

export const BackendStateProviderTesting = ({
  children,
  state,
}: {
  children: ComponentChildren;
  state: typeof defaultState;
}): VNode => {
  const value: BackendStateHandler = {
    state,
    logIn: () => {},
    expired: () => {},
    logOut: () => {},
  };

  return h(Context.Provider, {
    value,
    children,
  });
};
