/*
 This file is part of GNU Taler
 (C) 2021-2023 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 *
 * @author Sebastian Javier Marchano (sebasjm)
 */

import { h, VNode, FunctionalComponent } from "preact";
import { ConfigContextProvider } from "../../../context/config.js";
import { DetailPage as TestedComponent } from "./DetailPage.js";

export default {
  title: "Pages/Instance/Detail",
  component: TestedComponent,
  argTypes: {
    onUpdate: { action: "onUpdate" },
    onBack: { action: "onBack" },
  },
};

function createExample<Props>(
  Internal: FunctionalComponent<Props>,
  props: Partial<Props>,
) {
  const component = (args: any) => (
    <ConfigContextProvider
      value={{
        currency: "TESTKUDOS",
        version: "1",
        currencies: {
          currency: "TESTKUDOS"
        },
        exchanges: [],
        name: "taler-merchant"
      }}
    >
      <Internal {...(props as any)} />
    </ConfigContextProvider>
  );
  return { component, props };
}

export const Example = createExample(TestedComponent, {
  selected: {
    name: "name",
    auth: { method: "external" },
    address: {},
    user_type: "business",
    jurisdiction: {},
    use_stefan: true,
    default_pay_delay: {
      d_us: 1000 * 1000, //one second
    },
    default_wire_transfer_delay: {
      d_us: 1000 * 1000, //one second
    },
    merchant_pub: "ASDWQEKASJDKSADJ",
  },
});
